@@ -462,10 +462,78 @@ static const char *readbline(int fd, char *buf, int bufsize, int filesize, int *
     }
 }
 
+static unsigned int fmt_base64(char *dest, const char *src, unsigned int len)
+{
+	unsigned short bits = 0,temp = 0;
+	unsigned long written = 0;
+	unsigned int i;
+	const char base64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
+
+
+    for(i = 0; i < len; i++) {
+	temp <<= 8;
+	temp += src[i];
+	bits += 8;
+	while(bits > 6) {
+	    dest[written] = base64[((temp >> (bits - 6)) & 63)];
+	    written++;
+	    bits -= 6;
+	}
+    }
+
+    if(bits) {
+	temp <<= (6 - bits);
+	dest[written] = base64[temp & 63];
+	written++;
+    }
+
+    while(written & 3) {
+	dest[written] = '=';
+	written++;
+    }
+
+    return written;
+}
+
+static char *proxyauth(const char *user, const char *pass)
+{
+	int len;
+	char *buf, *userpass, *auth;
+
+
+    userpass = malloc(strlen(user) + strlen(pass) + 2);
+    if(!userpass) {
+	logg("!proxyauth: Can't allocate memory for 'userpass'\n");
+	return NULL;
+    }
+    sprintf(userpass, "%s:%s", user, pass);
+
+    buf = malloc((strlen(pass) + strlen(user)) * 2 + 4);
+    if(!buf) {
+	logg("!proxyauth: Can't allocate memory for 'buf'\n");
+	free(userpass);
+	return NULL;
+    }
+
+    len = fmt_base64(buf, userpass, strlen(userpass));
+    free(userpass);
+    buf[len] = '\0';
+    auth = malloc(strlen(buf) + 30);
+    if(!auth) {
+	free(buf);
+	logg("!proxyauth: Can't allocate memory for 'authorization'\n");
+	return NULL;
+    }
+
+    sprintf(auth, "Proxy-Authorization: Basic %s\r\n", buf);
+    free(buf);
+
+    return auth;
+}
+
 /*
  * TODO:
  * - strptime() is most likely not portable enough
- * - proxy support
  */
 int submitstats(const char *clamdcfg, const struct cfgstruct *copt)
 {
@@ -474,14 +542,14 @@ int submitstats(const char *clamdcfg, const struct cfgstruct *copt)
 	char query[SUBMIT_MIN_ENTRIES * 256];
 	char buff[512], statsdat[512], newstatsdat[512], uastr[128];
 	char logfile[256], fbuff[FILEBUFF];
-	char *pt, *pt2;
-	const char *line, *country = NULL;
+	char *pt, *pt2, *auth = NULL;
+	const char *line, *country = NULL, *user, *proxy = NULL;
 	struct cfgstruct *clamdopt;
 	const struct cfgstruct *cpt;
 	struct stat sb;
 	struct tm tms;
 	time_t epoch;
-	unsigned int qcnt, entries, submitted = 0, permfail = 0;
+	unsigned int qcnt, entries, submitted = 0, permfail = 0, port = 0;
 
 
     if((cpt = cfgopt(copt, "DetectionStatsCountry"))->enabled) {
@@ -558,6 +626,31 @@ int submitstats(const char *clamdcfg, const struct cfgstruct *copt)
 	snprintf(uastr, sizeof(uastr), PACKAGE"/%s (OS: "TARGET_OS_TYPE", ARCH: "TARGET_ARCH_TYPE", CPU: "TARGET_CPU_TYPE")%s%s", get_version(), country ? ":" : "", country ? country : "");
     uastr[sizeof(uastr) - 1] = 0;
 
+    if((cpt = cfgopt(copt, "HTTPProxyServer"))->enabled) {
+	proxy = cpt->strarg;
+	if(!strncasecmp(proxy, "http://", 7))
+	    proxy += 7;
+
+	if((cpt = cfgopt(copt, "HTTPProxyUsername"))->enabled) {
+	    user = cpt->strarg;
+	    if(!(cpt = cfgopt(copt, "HTTPProxyPassword"))->enabled) {
+		logg("!SubmitDetectionStats: HTTPProxyUsername requires HTTPProxyPassword\n");
+		close(fd);
+		return 56;
+	    }
+	    auth = proxyauth(user, cpt->strarg);
+	    if(!auth) {
+		close(fd);
+		return 56;
+	    }
+	}
+
+	if((cpt = cfgopt(copt, "HTTPProxyPort"))->enabled)
+	    port = cpt->numarg;
+
+	logg("*Connecting via %s\n", proxy);
+    }
+
     ret = 0;
     memset(query, 0, sizeof(query));
     qcnt = 0;
@@ -609,7 +702,7 @@ int submitstats(const char *clamdcfg, const struct cfgstruct *copt)
 	entries++;
 
 	if(entries == SUBMIT_MIN_ENTRIES) {
-	    sd = wwwconnect("stats.clamav.net", NULL, 0, NULL, cfgopt(copt, "LocalIPAddress")->strarg, cfgopt(copt, "ConnectTimeout")->numarg, NULL, 0, 0);
+	    sd = wwwconnect("stats.clamav.net", proxy, port, NULL, cfgopt(copt, "LocalIPAddress")->strarg, cfgopt(copt, "ConnectTimeout")->numarg, NULL, 0, 0);
 	    if(sd == -1) {
 		logg("!SubmitDetectionStats: Can't connect to server\n");
 		ret = 52;
@@ -618,13 +711,13 @@ int submitstats(const char *clamdcfg, const struct cfgstruct *copt)
 
 	    query[sizeof(query) - 1] = 0;
 	    snprintf(post, sizeof(post),
-		"POST /submit.php HTTP/1.0\r\n"
-		"Host: stats.clamav.net\r\n"
+		"POST http://stats.clamav.net/submit.php HTTP/1.0\r\n"
+		"Host: stats.clamav.net\r\n%s"
 		"Content-Type: application/x-www-form-urlencoded\r\n"
 		"User-Agent: %s\r\n"
 		"Content-Length: %u\r\n\n"
 		"%s",
-	    uastr, (unsigned int) strlen(query), query);
+	    auth ? auth : "", uastr, (unsigned int) strlen(query), query);
 
 	    if(send(sd, post, strlen(post), 0) < 0) {
 		logg("!SubmitDetectionStats: Can't write to socket\n");
@@ -665,14 +758,21 @@ int submitstats(const char *clamdcfg, const struct cfgstruct *copt)
 	    }
 
 	    ret = 52;
-	    if(strstr(post, "SUBMIT_PERMANENT_FAILURE")) {
+	    if((pt = strstr(post, "SUBMIT_PERMANENT_FAILURE"))) {
 		if(!submitted) {
-		    logg("!SubmitDetectionStats: Permanent failure\n");
 		    permfail = 1;
+		    if((pt + 32 <= post + sizeof(post)) && pt[24] == ':')
+			logg("!SubmitDetectionStats: Remote server reported permanent failure: %s\n", &pt[25]);
+		    else
+			logg("!SubmitDetectionStats: Remote server reported permanent failure\n");
+		}
+	    } else if((pt = strstr(post, "SUBMIT_TEMPORARY_FAILURE"))) {
+		if(!submitted) {
+		    if((pt + 32 <= post + sizeof(post)) && pt[24] == ':')
+			logg("!SubmitDetectionStats: Remote server reported temporary failure: %s\n", &pt[25]);
+		    else
+			logg("!SubmitDetectionStats: Remote server reported temporary failure\n");
 		}
-	    } else if(strstr(post, "SUBMIT_TEMPORARY_FAILURE")) {
-		if(!submitted)
-		    logg("!SubmitDetectionStats: Temporary failure\n");
 	    } else {
 		if(!submitted)
 		    logg("!SubmitDetectionStats: Incorrect answer from server\n");
@@ -684,6 +784,8 @@ int submitstats(const char *clamdcfg, const struct cfgstruct *copt)
     } while((line = readbline(fd, fbuff, FILEBUFF, sb.st_size, &lread)));
 
     close(fd);
+    if(auth)
+	free(auth);
 
     if(submitted || permfail) {
 	if((fd = open("stats.dat", O_WRONLY | O_CREAT | O_TRUNC, 0600)) == -1) {
@@ -713,76 +815,7 @@ static int Rfc2822DateTime(char *buf, time_t mtime)
     return strftime(buf, 36, "%a, %d %b %Y %X GMT", gmt);
 }
 
-static unsigned int fmt_base64(char *dest, const char *src, unsigned int len)
-{
-	unsigned short bits = 0,temp = 0;
-	unsigned long written = 0;
-	unsigned int i;
-	const char base64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
-
-
-    for(i = 0; i < len; i++) {
-	temp <<= 8;
-	temp += src[i];
-	bits += 8;
-	while(bits > 6) {
-	    dest[written] = base64[((temp >> (bits - 6)) & 63)];
-	    written++;
-	    bits -= 6;
-	}
-    }
-
-    if(bits) {
-	temp <<= (6 - bits);
-	dest[written] = base64[temp & 63];
-	written++;
-    }
-
-    while(written & 3) {
-	dest[written] = '=';
-	written++;
-    }
-
-    return written;
-}
-
-static char *proxyauth(const char *user, const char *pass)
-{
-	int len;
-	char *buf, *userpass, *auth;
-
-
-    userpass = malloc(strlen(user) + strlen(pass) + 2);
-    if(!userpass) {
-	logg("!proxyauth: Can't allocate memory for 'userpass'\n");
-	return NULL;
-    }
-    sprintf(userpass, "%s:%s", user, pass);
-
-    buf = malloc((strlen(pass) + strlen(user)) * 2 + 4);
-    if(!buf) {
-	logg("!proxyauth: Can't allocate memory for 'buf'\n");
-	free(userpass);
-	return NULL;
-    }
-
-    len = fmt_base64(buf, userpass, strlen(userpass));
-    free(userpass);
-    buf[len] = '\0';
-    auth = malloc(strlen(buf) + 30);
-    if(!auth) {
-	free(buf);
-	logg("!proxyauth: Can't allocate memory for 'authorization'\n");
-	return NULL;
-    }
-
-    sprintf(auth, "Proxy-Authorization: Basic %s\r\n", buf);
-    free(buf);
-
-    return auth;
-}
-
-static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int *ims, int ctimeout, int rtimeout, struct mirdat *mdat, int logerr, unsigned int can_whitelist)
+static struct cl_cvd *remote_cvdhead(const char *cvdfile, const char *localfile, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int *ims, int ctimeout, int rtimeout, struct mirdat *mdat, int logerr, unsigned int can_whitelist)
 {
 	char cmd[512], head[513], buffer[FILEBUFF], ipaddr[46], *ch, *tmp;
 	int bread, cnt, sd;
@@ -790,7 +823,6 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
 	char *remotename = NULL, *authorization = NULL;
 	struct cl_cvd *cvd;
 	char last_modified[36], uastr[128];
-	struct stat sb;
 
 
     if(proxy) {
@@ -810,8 +842,16 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
 	}
     }
 
-    if(stat(file, &sb) != -1 && sb.st_mtime < time(NULL)) {
-	Rfc2822DateTime(last_modified, sb.st_mtime);
+    if(!access(localfile, R_OK)) {
+	cvd = cl_cvdhead(localfile);
+	if(!cvd) {
+	    logg("!remote_cvdhead: Can't parse file %s\n", localfile);
+	    free(remotename);
+	    free(authorization);
+	    return NULL;
+	}
+	Rfc2822DateTime(last_modified, (time_t) cvd->stime);
+	cl_cvdfree(cvd);
     } else {
 	    time_t mtime = 1104119530;
 
@@ -821,7 +861,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
 
     logg("*If-Modified-Since: %s\n", last_modified);
 
-    logg("Reading CVD header (%s): ", file);
+    logg("Reading CVD header (%s): ", cvdfile);
 
     if(uas)
 	strncpy(uastr, uas, sizeof(uastr));
@@ -836,7 +876,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
 	"Connection: close\r\n"
 	"Range: bytes=0-511\r\n"
         "If-Modified-Since: %s\r\n"
-        "\r\n", (remotename != NULL) ? remotename : "", file, hostname, (authorization != NULL) ? authorization : "", uastr, last_modified);
+        "\r\n", (remotename != NULL) ? remotename : "", cvdfile, hostname, (authorization != NULL) ? authorization : "", uastr, last_modified);
 
     free(remotename);
     free(authorization);
@@ -852,7 +892,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
 	return NULL;
     } else {
 	logg("*Connected to %s (IP: %s).\n", hostname, ipaddr);
-	logg("*Trying to retrieve CVD header of http://%s/%s\n", hostname, file);
+	logg("*Trying to retrieve CVD header of http://%s/%s\n", hostname, cvdfile);
     }
 
     if(!ip[0])
@@ -1505,7 +1545,7 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 
     if(!nodb && !newver) {
 
-	remote = remote_cvdhead(cvdfile, hostname, ip, localip, proxy, port, user, pass, uas, &ims, ctimeout, rtimeout, mdat, logerr, can_whitelist);
+	remote = remote_cvdhead(cvdfile, localname, hostname, ip, localip, proxy, port, user, pass, uas, &ims, ctimeout, rtimeout, mdat, logerr, can_whitelist);
 
 	if(!nodb && !ims) {
 	    logg("%s is up to date (version: %d, sigs: %d, f-level: %d, builder: %s)\n", localname, current->version, current->sigs, current->fl, current->builder);
